// libTorrent - BitTorrent library
// Copyright (C) 2005-2011, Jari Sundell
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// In addition, as a special exception, the copyright holders give
// permission to link the code of portions of this program with the
// OpenSSL library under certain conditions as described in each
// individual source file, and distribute linked combinations
// including the two.
//
// You must obey the GNU General Public License in all respects for
// all of the code used other than OpenSSL.  If you modify file(s)
// with this exception, you may extend this exception to your version
// of the file(s), but you are not obligated to do so.  If you do not
// wish to do so, delete this exception statement from your version.
// If you delete this exception statement from all source files in the
// program, then also delete it here.
//
// Contact:  Jari Sundell <jaris@ifi.uio.no>
//
//           Skomakerveien 33
//           3185 Skoppum, NORWAY

#include "config.h"

#include <iostream>

#include "torrent/exceptions.h"
#include "http.h"

namespace torrent {

template <typename Slot>
inline void
slot_list_call(const std::list<Slot>& slot_list) {
  if (slot_list.empty())
    return;

  typename std::list<Slot>::const_iterator first = slot_list.begin();
  typename std::list<Slot>::const_iterator next = slot_list.begin();

  while (++next != slot_list.end()) {
    (*first)();
    first = next;
  }

  (*first)();
}

template <typename Slot, typename Arg1>
inline void
slot_list_call(const std::list<Slot>& slot_list, Arg1 arg1) {
  if (slot_list.empty())
    return;

  typename std::list<Slot>::const_iterator first = slot_list.begin();
  typename std::list<Slot>::const_iterator next = slot_list.begin();

  while (++next != slot_list.end()) {
    (*first)(arg1);
    first = next;
  }

  (*first)(arg1);
}

Http::slot_factory Http::m_factory;

Http::~Http() {
}

Http*
Http::call_factory() {
  if (!m_factory)
    throw internal_error("Http factory not set.");

  Http* h = m_factory();

  if (h == NULL)
    throw internal_error("Http factory returned a NULL object.");

  return h;
}

void
Http::trigger_done() {
  torrent::slot_list_call(signal_done());

  if (m_flags & flag_delete_stream) {
    delete m_stream;
    m_stream = NULL;
  }

  if (m_flags & flag_delete_self)
    delete this;
}

void
Http::trigger_failed(const std::string& message) {
  torrent::slot_list_call(signal_failed(), message);

  if (m_flags & flag_delete_stream) {
    delete m_stream;
    m_stream = NULL;
  }

  if (m_flags & flag_delete_self)
    delete this;
}

}

